#!/usr/bin/python
"""
ApmInfo - Power info utility

Copyright (C) 2001 James ''Wez''Weatherall

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""

try: import pylau
except ImportError: pass

import gtk
import sys
import string

def gdk_intern_atom(name):
    import _gtk
    return _gtk.gdk_atom_intern(name, 1)

def set_gdk_win_prop(window, property, type,
                     format, mode, data):
    import _gtk, GDK
    atom = _gtk.gdk_atom_intern(property, 1)
    if not int(atom): return 0
    int_atom = _gtk.gdk_atom_intern(type, 1)
    if not int(int_atom): return 0
    if mode=="REPLACE":gdk_mode = GDK.PROP_MODE_REPLACE
    elif mode=="APPEND":gdk_mode = GDK.PROP_MODE_APPEND
    elif mode=="PREPEND":gdk_mode = GDK.PROP_MODE_PREPEND
    try:
        window.property_change(atom, int_atom, format, gdk_mode, data)
        return 1
    except: return 0

class ApmInfo(gtk.GtkWindow):
    def __init__(self, dock):
        self.__dock = dock
        gtk.GtkWindow.__init__(self)
        self.set_name("ApmInfo")
        self.set_title("ApmInfo")
        if dock:
            self.__power = gtk.GtkButton()
	    self.__power.connect("clicked", self.OnClicked)
            icon = gtk.GtkPixmap(self, "/usr/X11R6/include/X11/pixmaps/apminfo.xpm")
            self.__power.add(icon)
            self.add(self.__power)
        else:
            self.__power = gtk.GtkProgressBar()
            self.__power.set_show_text(1)
            frame = gtk.GtkAspectFrame("Power", 0.5, 0.5, 0,1)
            frame.add(self.__power)
            self.add(frame)
            self.__power.set_format_string("%p%%")
        self.connect("destroy", self.OnDestroy)
        self.realize()
        if dock:
            set_gdk_win_prop(self.get_window(), "_NET_WM_WINDOW_TYPE",
                             "ATOM", 32, "REPLACE",
                             [gdk_intern_atom("_NET_WM_WINDOW_TYPE_DOCK")])
        self.set_default_size(8, 8)
        self.show_all()
        self.OnTimeout()
        self.__timeout = gtk.timeout_add(4000, self.OnTimeout)
    def OnDestroy(self, widget):
        if self.__timeout:gtk.timeout_remove(self.__timeout)
        gtk.mainquit()
    def OnClicked(self, widget):
	# - Should present a window with full APM status & Quit option
	self.destroy()
    def OnTimeout(self):
        apminfo = open("/proc/apm", "r").read()
        battery = string.split(apminfo)[6]
        if battery[len(battery)-1] == "%":
	    midlevel = 65.0
            level = max(0, min(100, eval(battery[:-1])))
            if self.__dock:
                # *** Does this leak styles???
                style = self.__power.get_style().copy()
                red = 65535*max(0.0, min(1.0, (100.0-level)/(100.0-midlevel)))
                green = 65535*max(0.0, min(1.0, (level/midlevel)))
                if red>32767: red = red-65536
                if green>32767: green = green-65536
                style.bg[gtk.STATE_NORMAL] = gtk.GdkColor(red,
                                                          green,
                                                          0)
                self.__power.set_style(style)
            else: self.__power.update(max(0.0, min(1.0, level/100.0)))
            return 1

level = 0

def Main(argv):
    dock = 0
    if "--dock" in argv: dock=1
    meminfo = ApmInfo(dock)
    gtk.mainloop()

if __name__ == "__main__": Main(sys.argv)
